import { Grammar } from "../grammar.js";
import { tmrColors, tmrConstants, tmrTokenZIndex, TMRUtility } from "../tmr-utility.js";
import { Draconique } from "./draconique.js";

export class ReserveExtensible extends Draconique {
  constructor() {
    super();
  }

  type() { return 'tete' }
  match(item) { return Draconique.isTeteDragon(item) && Grammar.toLowerCaseNoAccent(item.name).includes("reserve extensible"); }
  manualMessage() { return "Vous pouvez re-configurer votre Réserve extensible" }
  async onActorCreateOwned(actor, tete) {
    const existants = actor.data.items.filter(it => this.isCase(it)).map(it => it.data.coord);
    const tmr = TMRUtility.getTMRAleatoire(it => !(it.type == 'fleuve' || existants.includes(it.coord)));
    await this.createCaseTmr(actor, "Nouvelle Réserve extensible", tmr, tete.id);
  }

  code() { return 'reserve_extensible' }
  tooltip(linkData) { return `Réserve extensible en ${this.tmrLabel(linkData)} !` }
  img() { return 'systems/foundryvtt-reve-de-dragon/icons/tmr/treasure-chest.svg' }

  createSprite(pixiTMR) {
    return pixiTMR.sprite(this.code(), {
      zIndex: tmrTokenZIndex.tetes,
      alpha: 0.9,
      decallage: tmrConstants.left
    });
  }

}
